#include <Wire.h> 
#include <LCD.h>
#include <LiquidCrystal_I2C.h>

#define I2C_ADDR    0x3F  // Define I2C Address where the PCF8574A is

#define BACKLIGHT_PIN     3
#define En_pin  2
#define Rw_pin  1
#define Rs_pin  0
#define D4_pin  4
#define D5_pin  5
#define D6_pin  6
#define D7_pin  7

#define but_1   4
#define but_2   5
#define but_3   6 

#define DLI     3
LiquidCrystal_I2C       lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);

void setup() {
  pinMode(but_1, INPUT_PULLUP);
  pinMode(but_2, INPUT_PULLUP);
  pinMode(but_3, INPUT_PULLUP);
  pinMode(DLI, OUTPUT);
  Serial.begin(9600);
  lcd.begin (20,4,LCD_5x8DOTS);
  lcd.setBacklightPin(BACKLIGHT_PIN,POSITIVE); // init the backlight
  lcd.setBacklight(HIGH);
  lcd.print("init");
}

int timeToSleep = 60;
int timeInactive = 0;
int lastTimeActive = millis() / 1000;
String powerStatus = "off";
int endT = startTimer(0);
int timeLeft = endT - (millis()/1000);
bool sleep = false;
bool b1_prev = 1;
bool b2_prev = 1;
bool b3_prev = 1;

bool power_on = false;

void loop() {
  timeInactive = millis()/1000 - lastTimeActive;
  if (digitalRead(but_1) == 0 or digitalRead(but_2) == 0 or digitalRead(but_3) == 0){
    lastTimeActive = millis() / 1000;
  }

  // put your main code here, to run repeatedly:
  timeLeft = endT - (millis()/1000);
  if (timeLeft < 0){
    endT = millis()/1000; 
  }
  
  if (buttonPressed(b1_prev, but_1)){
    Serial.println("button 1 pressed");
    endT = millis() / 1000;
  }
  b1_prev = digitalRead(but_1);

  if (buttonPressed(b2_prev, but_2)){
    Serial.println("button 2 pressed");
    endT = endT - (60 * 15);
  }
  b2_prev = digitalRead(but_2);

  if (buttonPressed(b3_prev, but_3)){
    Serial.println("button 3 pressed");
    endT = endT +(60 * 15);
  }
  b3_prev = digitalRead(but_3);
  if (timeLeft > 5 * 60 * 60){
    endT = (5 * 60 * 60) + (millis()/1000);
  }
  if (timeLeft > 0){ 
    powerStatus = "on ";
    timeLeft = endT - (millis()/1000);
    lcd.setCursor(10,0);
    lcd.print(powerStatus);
    lcd.setCursor(10,1);
    lcd.print("---");
  }
  else if (not sleep){
    powerStatus = "off";
    lcd.setCursor(10, 1);
    lcd.print(powerStatus);  
    lcd.setCursor(10,0);
    lcd.print("---"); 
    timeLeft = 0;
  }
  if (not sleep){
    lcd.setBacklight(HIGH);
    printTime(timeLeft);
  }
  //Serial.print(digitalRead(but_1));
  //Serial.print("\t");
  //Serial.print(digitalRead(but_2));
  //Serial.print("\t");
  //Serial.print(digitalRead(but_3));
  //Serial.print("\t");
  //Serial.println();
  power_on = (timeLeft > 0);
  sleep = false;
  if (power_on){
    digitalWrite(DLI, HIGH);
  }
  else{
    digitalWrite(DLI, LOW);
    if (timeInactive > timeToSleep){
      displayOff();
      sleep = true;
    }
  }
}

void printTime(int o_seconds)
{
  int hours = floor(o_seconds / 3600);
  int minutes = floor((o_seconds - (hours*3600))/60);
  int seconds = o_seconds - minutes * 60 - hours * 3600;
  Serial.print(hours);
  Serial.print("\t");
  Serial.print(minutes);
  Serial.print("\t");
  Serial.print(seconds);
  Serial.print("\t");
  Serial.println();
  //lcd.clear();
  
  lcd.setCursor(0,0);
  lcd.print(hours);
  lcd.print(":");
  if (minutes < 10){
    lcd.print("0");  
  }
  lcd.print(minutes);
  lcd.print(":");
  if (seconds < 10){
    lcd.print("0");
  }
  lcd.print(seconds);
  lcd.print("   ");
}

int startTimer(int seconds)
{
  int startT = millis()/1000;
  int endT = startT + seconds;
  Serial.println(endT);
  return(endT);
}

bool buttonPressed(bool previous_status,int button_id){
  if (previous_status == 1){
    if (digitalRead(button_id) == 0){
      return true;
    }
    else{
      return false;
    }
  }
  else{
    return false;
  }
}

void displayOff(){
  lcd.clear();
  lcd.setBacklight(LOW);
}
